/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/

// Ray test and identify triangles demo.
// mihail.isakov@gmail.com

#ifndef HITTRIANGLEDEMO_H
#define HITTRIANGLEDEMO_H

#include "BulletCollision/Gimpact/btGImpactShape.h"
#include "BulletCollision/Gimpact/btGImpactCollisionAlgorithm.h"

#include "GlutDemoApplication.h"

ATTRIBUTE_ALIGNED16(class) HitTrack
{

public:

BT_DECLARE_ALIGNED_ALLOCATOR();

	HitTrack() : id(0),
				 subpart(0),
				 timestamp(0),
				 triangle_exists(0),
				 distance(0)
	{
		shape	   = 0;
		rigid_body  = 0;
	}
	
	~HitTrack()
	{
	}

	void set_shape(btCollisionShape * s)
	{
		shape = s;
	}

	btCollisionShape * get_shape()
	{
		return shape;
	}

	void set_rigid_body(btRigidBody * rb)
	{
		rigid_body = rb;
	}

	btRigidBody * get_rigid_body()
	{
		return rigid_body;
	}

	unsigned int get_id()
	{
		return id;
	}

	void set_id(unsigned int i)
	{
		id = i;
	}

	void set_position(float * p)
	{
		position[0] = p[0];
		position[1] = p[1];
		position[2] = p[2];
	}

	float * get_position()
	{
		return position;
	}

	void set_triangle(float * t)
	{
		triangle[0] = t[0];
		triangle[1] = t[1];
		triangle[2] = t[2];
		triangle[3] = t[3];
		triangle[4] = t[4];
		triangle[5] = t[5];
		triangle[6] = t[6];
		triangle[7] = t[7];
		triangle[8] = t[8];
	}

	float * get_triangle()
	{
		return triangle;
	}

	unsigned long int get_timestamp()
	{
		return timestamp;
	}

	void set_timestamp(unsigned long int t)
	{
		timestamp = t;
	}

	float get_distance()
	{
		return distance;
	}

	void set_distance(float d)
	{
		distance = d;
	}

	int get_subpart()
	{
		return subpart;
	}

	void set_subpart(int s)
	{
		subpart = s;
	}

unsigned int id;
int subpart;
unsigned long int timestamp;
short int triangle_exists;
float distance;
float position[3];
float triangle[9];
btCollisionShape * shape;
btRigidBody * rigid_body;
};

class HitTriangleDemo : public GlutDemoApplication
{
	public:

	btAlignedObjectArray<btCollisionShape*> m_collisionShapes;

	class btBroadphaseInterface*	m_overlappingPairCache;

	class btCollisionDispatcher*	m_dispatcher;

	class btConstraintSolver*	m_constraintSolver;

	class btDefaultCollisionConfiguration* m_collisionConfiguration;

	class btTriangleIndexVertexArray * m_ground_array;

	btVector3 * m_ground_vertices;

	HitTriangleDemo();

	virtual ~HitTriangleDemo();

	virtual void clientMoveAndDisplay();

	virtual void keyboardCallback(unsigned char key, int x, int y);

	virtual void mouseFunc(int button, int state, int x, int y);

	void initPhysics();

	static DemoApplication* Create()
	{
		HitTriangleDemo* demo = new HitTriangleDemo();
		demo->myinit();
		demo->initPhysics();
		return demo;
	}

	bool process_triangle(btCollisionShape * shape,
						  int hitTriangleIndex,
						  float * triangle);

	btAlignedObjectArray<HitTrack *> hit_tracks;

};

#endif //HITTRIANGLEDEMO_H


